#include "NamedCounterFactory.h"

using std::string;

NamedCounterFactoryImpl::NamedCounterFactoryImpl(const size_t numCounters)
{
   size_t value = 0;

   for (int w = 'A'; w <= 'Z'; w++)
   {
      for (int x = 'A'; x <= 'Z'; x++)
      {
         for (int y = 'A'; y <= 'Z'; y++)
         {
            for (int z = 'A'; z <= 'Z'; z++)
            {
               if (value == numCounters)
               {
                  return;
               }

               char name[5] = { w, x, y, z, '\0' };
            
               AddNamedCounter(name, value++);
            }
         }
      }
   }
}

NamedCounterFactoryImpl::~NamedCounterFactoryImpl()
{
   CounterContainer::iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      (*it).second->Release();
      ++it;
   }
}

NamedCounter_ptr NamedCounterFactoryImpl::GetNamedCounter(
   const char* name)
{
   CounterContainer::const_iterator it = m_counters.find(string(name));

   if (it == m_counters.end())
   {
      // not found

      throw NamedCounterFactory::NoCounterExists(name);
   }

   (*it).second->AddRef();

   return NamedCounter::_duplicate((*it).second->_this());
}

void NamedCounterFactoryImpl::AddNamedCounter(
   const char* name, 
   CORBA::Long initialValue)
{
   CounterContainer::const_iterator it = m_counters.find(string(name));

   if (it != m_counters.end())
   {
      // found

      throw NamedCounterFactory::CounterExists(name);
   }

   NamedCounterImpl *pCounter = new NamedCounterImpl(name, initialValue);

   m_counters.insert(CounterContainer::value_type(name, pCounter));
}

void NamedCounterFactoryImpl::RemoveNamedCounter(
   const char* name)
{
   CounterContainer::iterator it = m_counters.find(string(name));

   if (it == m_counters.end())
   {
      // not found

      throw NamedCounterFactory::NoCounterExists(name);
   }

   // this erases the map entry between the name of the counter and
   // the pointer to the counter object, unfortunately we can't delete
   // the counter object itself as to do so would cause  

   (*it).second->Release();

   m_counters.erase(it);   
}
  

NamedCounterFactory::counterSeq *NamedCounterFactoryImpl::GetAll()
{
   const size_t numCounters = m_counters.size();


   NamedCounterFactory::counterSeq_var seq = new NamedCounterFactory::counterSeq(numCounters);

	seq->length(numCounters);

   size_t i = 0;

   CounterContainer::const_iterator it = m_counters.begin();
   
   while (it != m_counters.end())
   {
      (*it).second->AddRef();

      seq[i] = NamedCounter::_duplicate((*it).second->_this());
    
      ++it;
      ++i;
   }

   return seq._retn();
}
